unit ComprClasses;

interface

uses
  SysUtils, Classes, Zlib;

type

  {TComprMemStream is a TMemoryStream descendant that adds compression methods.
   The data in the buffer is not compressed.}
  TComprMemStream = class(TMemoryStream)
  private
    FComprLevel: integer;
  public
    constructor Create(ComprLevel: Integer);
    procedure DecomprFromFile(const FileName: string); virtual;
    procedure DecomprFromStream(Stream: TStream); virtual;
    procedure ComprToFile(const FileName: string); virtual;
    procedure ComprToStream(Stream: TStream); virtual;
  end;

  TComprFileStream = class(TComprMemStream)
  public
    constructor Create(const FileName: string; ComprLevel: integer);
    destructor Destroy; override;
  end;

  {TComprStrings is a TStringList descendant that adds compression methods.
   The data in the string list is not compressed.}
  TComprStrings = class(TStringList)
  private
    FComprLevel: integer;
  public
    constructor Create(ComprLevel: Integer);
    procedure DecomprFromFile(const FileName: string); virtual;
    procedure DecomprFromStream(Stream: TStream); virtual;
    procedure ComprToFile(const FileName: string); virtual;
    procedure ComprToStream(Stream: TStream); virtual;
  end;

  ECompression = class(Exception);

const
  {compression levels; values are actually -1..9}
  zLevelNone = Z_NO_COMPRESSION;           //  0
  zLevelFast = Z_BEST_SPEED;               //  1
  zLevelSmall = Z_BEST_COMPRESSION;        //  9
  zLevelDefault = Z_DEFAULT_COMPRESSION;   // -1

implementation

procedure zCheck(ErrCode : integer);
begin
  if ErrCode < 0 then
  begin
    case ErrCode of
      -1 : raise ECompression.Create('General compression library failure');
      -2 : raise ECompression.Create('Compression stream error');
      -3 : raise ECompression.Create('Compression data error');
      -4 : raise ECompression.Create('Compression memory error');
      -5 : raise ECompression.Create('Compression buffer error');
      -6 : raise ECompression.Create('Compression library version error');
    end;
  end;
end;

{TComprMemStream}

constructor TComprMemStream.Create(ComprLevel: Integer);
begin
  if ((ComprLevel + 1) in [0..10]) then
    FComprLevel := ComprLevel
  else
    FComprLevel := -1;
end;

procedure TComprMemStream.DecomprFromFile(const FileName: string);
var
  FStream: TStream;
begin
  FStream := TFileStream.Create(FileName, fmOpenRead);
  try
    DecomprFromStream(FStream);
  finally
    FStream.Free;
  end;
end;

procedure TComprMemStream.DecomprFromStream(Stream: TStream);
var
  ZStream: z_stream; //  compression stream record
  DataSize: Longint;
  ComprBuf : PChar;
  UnComprLen: Longint;  // same size as z_stream.total_in
begin
  with ZStream do
  begin
    zalloc := Z_NULL;
    zfree := Z_NULL;
    opaque := Z_NULL;
  end;

  zCheck(inflateInit(ZStream));  // allocate compression stream record

  Stream.Read(UnComprLen, sizeof(Longint));  // read in uncompressed size
  DataSize := Stream.Size - Stream.Position;  //size of compressed data in bytes

  ComprBuf := StrAlloc(DataSize); // allocate memory for uncompressed data

  try
    SetSize(UnComprLen);   // adjust size to uncompr size

    Stream.Read(ComprBuf^, DataSize);  // read compressed data into ComprBuf

    with ZStream do
    begin
      next_in := ComprBuf;     // compressed data
      avail_in := DataSize;    // comressed data size
      next_out := Memory;      // buffer for uncompressed data
      avail_out := Size;       // size of uncompressed data buffer
    end;

    zCheck(inflate(ZStream, Z_FINISH));  // decompress in one step
    zCheck(inflateEnd(ZStream));         // free compression stream record
  finally
    StrDispose(ComprBuf);
  end;
end;

procedure TComprMemStream.ComprToFile(const FileName: string);
var
  FStream : TStream; //VCL stream object
begin
  FStream := TFileStream.Create(FileName, fmCreate);
  try
    ComprToStream(FStream);
  finally
    FStream.Free;
  end;
end;

procedure TComprMemStream.ComprToStream(Stream: TStream);
var
  ZStream: z_stream; //compression stream record
  ComprBuf: PChar;
begin
  with ZStream do
  begin
    zalloc := Z_NULL;
    zfree := Z_NULL;
    opaque := Z_NULL;
  end;

  zCheck(deflateInit(ZStream, FComprLevel)); // create compression stream rec

  ComprBuf := StrAlloc(Size);  // allocate a safe amount of memory

  try
    // set up compression stream record
    with ZStream do
    begin
      next_out := ComprBuf;
      avail_out := Size;
      next_in := Memory;
      avail_in := Size;
    end;

    zCheck(deflate(ZStream, Z_FINISH));  // compress in one step

    Stream.Write(ZStream.total_in, sizeof(Longint));  // uncompressed length
    Stream.WriteBuffer(ComprBuf^, ZStream.total_out); // compressed data

    zCheck(deflateEnd(ZStream));  // free compression stream record
  finally
    StrDispose(ComprBuf);
  end;
end;

{TComprFileStream}

constructor TComprFileStream.Create(const FileName: string; ComprLevel: integer);
begin
end;

destructor TComprFileStream.Destroy;
begin
  inherited Destroy;
end;

{TComprStrings}

constructor TComprStrings.Create(ComprLevel: integer);
begin
  if not (ComprLevel in [0..9]) then
    FComprLevel := -1
  else
    FComprLevel := ComprLevel;
end;

procedure TComprStrings.DecomprFromFile(const FileName: string);
var
  FStream: TStream;
begin
  FStream := TFileStream.Create(FileName, fmOpenRead);
  try
    DecomprFromStream(FStream);
  finally
    FStream.Free;
  end;
end;

procedure TComprStrings.DecomprFromStream(Stream: TStream);
var
  ZStream: z_stream; //  compression stream record
  DataSize: Longint;
  S: string;
  ComprBuf : PChar;
  UncomprBuf : PChar;
  UnComprLen: Longint;  // same size as z_stream.total_in
begin
  with ZStream do
  begin
    zalloc := Z_NULL;
    zfree := Z_NULL;
    opaque := Z_NULL;
  end;

  zCheck(inflateInit(ZStream));  // allocate compression stream record

  BeginUpdate;

  Stream.Read(UnComprLen, sizeof(Longint));  // read in uncompressed size
  DataSize := Stream.Size - Stream.Position;  //size of compressed data in bytes

  ComprBuf := StrAlloc(DataSize);  // allocate memory for compressed data
  UncomprBuf := StrAlloc(UnComprLen + (UnComprLen div 1024) + 12);

  try
    SetString(S, nil, UnComprLen);   // initialize string length to uncompr size

    Stream.Read(ComprBuf^, DataSize);     // read compressed data into ComprBuf

    with ZStream do
    begin
      next_in := ComprBuf;  // compressed data
      avail_in := DataSize; // comressed data size
      next_out := UncomprBuf; // buffer for uncompressed data
      avail_out := UnComprLen +
                   (UnComprLen div 1024) + 12; // size of uncompressed data buf
    end;

    zCheck(inflate(ZStream, Z_FINISH));  // decompress in one step
    zCheck(inflateEnd(ZStream));         // free compression stream record

    UncomprBuf[ZStream.total_out] := #0;
    StrCopy(PChar(S), UncomprBuf);
    SetTextStr(S);  // set the strings to uncompressed data
  finally
    StrDispose(ComprBuf);
    StrDispose(UncomprBuf);
    EndUpdate;
  end;
end;

procedure TComprStrings.ComprToFile(const FileName: string);
var
  FStream : TStream; //VCL stream object
begin
  FStream := TFileStream.Create(FileName, fmCreate);
  try
    ComprToStream(FStream);
  finally
    FStream.Free;
  end;
end;

procedure TComprStrings.ComprToStream(Stream: TStream);
var
  ZStream: z_stream; //compression stream record
  FullText: PChar;
  FullLen: Longint;
  ComprBuf: PChar;
begin
  with ZStream do
  begin
    zalloc := Z_NULL;
    zfree := Z_NULL;
    opaque := Z_NULL;
  end;

  zCheck(deflateInit(ZStream, FComprLevel)); // create compression stream rec

  FullText := GetText;
  FullLen := StrLen(FullText);
  ComprBuf := StrAlloc(FullLen);  // allocate a safe amount of memory

  try
    // set up compression stream record
    with ZStream do
    begin
      next_out := ComprBuf;
      avail_out := FullLen;
      next_in := FullText;
      avail_in := FullLen;
    end;

    zCheck(deflate(ZStream, Z_FINISH));  // compress in one step

    Stream.Write(ZStream.total_in, sizeof(Longint));  // uncompressed length
    Stream.WriteBuffer(ComprBuf^, ZStream.total_out); // compressed data

    zCheck(deflateEnd(ZStream));  // free compression stream record
  finally
    StrDispose(ComprBuf);
  end;
end;

end.
