unit ComprDB;

interface

uses
  SysUtils, Windows, Classes, Graphics, ComprClasses, Zlib, DBTables;

type

  TComprBlobField = class(TField)
  private
    FComprLevel: Integer;
    procedure SetComprLevel(Value: Integer);
    function GetUncomprSize: Longint;
    procedure LoadFromBlob(Blob: TBlobField);
    procedure LoadFromBitmap(Bitmap: TBitmap);
    procedure LoadFromStrings(Strings: TStrings);
    procedure SaveToStrings(Strings: TStrings);
    procedure SaveToBitmap(Bitmap: TBitmap);
  protected
    function GetAsString: string; override;
    procedure SetAsString(const Value: string); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure LoadFromFile(const FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const FileName: string);
    procedure SaveToStream(Stream: TStream);
    property ComprLevel: Integer read FComprLevel write SetComprLevel;
    property UncomprSize: Longint read GetUncomprSize;
  end;

implementation

type
  TGraphicHeader = record
    Count: Word;                { Fixed at 1 }
    HType: Word;                { Fixed at $0100 }
    Size: Longint;              { Size not including header }
  end;

constructor TComprBlobField.Create(AOwner: TComponent);
begin
  { set default values and call inherited method }
  FComprLevel := zLevelDefault;
  inherited Create(AOwner);
end;

procedure TComprBlobField.SetComprLevel(Value: Integer);
begin
  { validate value then set property }
  if ((Value + 1) in [0..10]) then
    FComprLevel := Value;
end;

function TComprBlobField.GetUncomprSize: Longint;
begin
  { first 4 bytes of the blob data is uncompressed size }
  with TBlobStream.Create(Self, bmRead) do
    try
      { derefrence Result so that data is read to the right place }
      ReadBuffer(Pointer(Result)^, sizeof(Longint));
    finally
      Free;
    end;
end;

procedure TComprBlobField.LoadFromBlob(Blob: TBlobField);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
begin
  BlobStream := TBlobStream.Create(Self, bmWrite);
  ComprStream := TComprMemStream.Create(FComprLevel);
  try
    Blob.SaveToStream(ComprStream);
    ComprStream.Position := 0;
    ComprStream.ComprToStream(BlobStream);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

procedure TComprBlobField.LoadFromBitmap(Bitmap: TBitmap);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
  Header: TGraphicHeader;
begin
  BlobStream := TBlobStream.Create(Self, bmWrite);
  ComprStream := TComprMemStream.Create(FComprLevel);
  try
    if (DataType = ftGraphic) or (DataType = ftTypedBinary) then
    begin
      Header.Count := 1;
      Header.HType := $0100;
      Header.Size := 0;
      ComprStream.Write(Header, SizeOf(Header));
      Bitmap.SaveToStream(ComprStream);
      Header.Size := ComprStream.Position - SizeOf(Header);
      ComprStream.Position := 0;
      ComprStream.Write(Header, SizeOf(Header));
    end else
      Bitmap.SaveToStream(ComprStream);
      ComprStream.Position := 0;
      ComprStream.ComprToStream(BlobStream);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

procedure TComprBlobField.LoadFromStrings(Strings: TStrings);
var
  BlobStream: TBlobStream;
  ComprStrings: TComprStrings;
begin
  BlobStream := TBlobStream.Create(Self, bmWrite);
  ComprStrings := TComprStrings.Create(FComprLevel);
  try
    ComprStrings.Assign(Strings);
    ComprStrings.ComprToStream(BlobStream);
  finally
    BlobStream.Free;
    ComprStrings.Free;
  end;
end;

procedure TComprBlobField.SaveToStrings(Strings: TStrings);
var
  BlobStream: TBlobStream;
  ComprStrings: TComprStrings;
begin
  BlobStream := TBlobStream.Create(Self, bmRead);
  ComprStrings := TComprStrings.Create(zLevelDefault);
  try
    ComprStrings.DecomprFromStream(BlobStream);
    Strings.Assign(ComprStrings);
  finally
    BlobStream.Free;
    ComprStrings.Free;
  end;
end;

procedure TComprBlobField.SaveToBitmap(Bitmap: TBitmap);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
  DataSize: Longint;
  Header: TGraphicHeader;
begin
  BlobStream := TBlobStream.Create(Self, bmRead);
  ComprStream := TComprMemStream.Create(zLevelDefault);
  try
    ComprStream.DecomprFromStream(BlobStream);
    DataSize := ComprStream.Size;
    if DataSize >= SizeOf(TGraphicHeader) then
    begin
      ComprStream.Position := 0;
      ComprStream.Read(Header, SizeOf(Header));
      if (Header.Count <> 1) or (Header.HType <> $0100) or
        (Header.Size <> DataSize - SizeOf(Header)) then
        ComprStream.Position := 0;
    end;
    BitMap.LoadFromStream(ComprStream);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

function TComprBlobField.GetAsString: string;
var
  BlobStream: TBlobStream;
  ComprStrings: TComprStrings;
begin
  BlobStream := TBlobStream.Create(Self, bmRead);
  ComprStrings := TComprStrings.Create(FComprLevel);
  try
    ComprStrings.DecomprFromStream(BlobStream);
    Result := ComprStrings.Text;
  finally
    BlobStream.Free;
    ComprStrings.Free;
  end;
end;

procedure TComprBlobField.SetAsString(const Value: string);
var
  BlobStream: TBlobStream;
begin
  BlobStream := TBlobStream.Create(Self, bmWrite);
  with TComprMemStream.Create(zLevelDefault) do
    try
      Write(Pointer(Value)^, Length(Value));
      Position := 0;
      ComprToStream(BlobStream);
    finally
      BlobStream.Free;
      Free;
    end;
end;

procedure TComprBlobField.LoadFromFile(const FileName: string);
var
  ComprStream: TComprMemStream;
  BlobStream: TBlobStream;
begin
  ComprStream := TComprMemStream.Create(FComprLevel);
  BlobStream := TBlobStream.Create(Self, bmWrite);
  try
    ComprStream.LoadFromFile(FileName);
    ComprStream.Position := 0;
    ComprStream.ComprToStream(BlobStream);
  finally
    ComprStream.Free;
    BlobStream.Free;
  end;
end;

procedure TComprBlobField.LoadFromStream(Stream: TStream);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
begin
  BlobStream := TBlobStream.Create(Self, bmWrite);
  ComprStream := TComprMemStream.Create(FComprLevel);
  try
    ComprStream.LoadFromStream(Stream);
    ComprStream.Position := 0;
    ComprStream.ComprToStream(BlobStream);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

procedure TComprBlobField.SaveToFile(const FileName: String);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
begin
  BlobStream := TBlobStream.Create(Self, bmRead);
  ComprStream := TComprMemStream.Create(FComprLevel);
  try
    ComprStream.DecomprFromStream(BlobStream);
    ComprStream.Position := 0;
    ComprStream.SaveToFile(FileName);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

procedure TComprBlobField.SaveToStream(Stream: TStream);
var
  BlobStream: TBlobStream;
  ComprStream: TComprMemStream;
begin
  BlobStream := TBlobStream.Create(Self, bmRead);
  ComprStream := TComprMemStream.Create(FComprLevel);
  try
    ComprStream.DecomprFromStream(BlobStream);
    ComprStream.SaveToStream(Stream);
  finally
    BlobStream.Free;
    ComprStream.Free;
  end;
end;

end.
