unit TestProcs;

interface

const

  hello : PChar = 'hello, hello!';
///* "hello world" would be more standard, but the repeated "hello"
// * stresses the compression code better, sorry...
// */

  dictionary : PChar = 'hello';

var
  dictId : longint; // /* Adler32 value of the dictionary */

  Gcompr : PChar;
  Guncompr : PChar;
  GcomprLen : longint;
  GuncomprLen : longint;


procedure test_compress(compr: PChar; comprLen: longint;
		            uncompr : PChar; uncomprLen: longint);

procedure test_gzio(outname, inname : PChar;
		            uncompr : PChar; uncomprLen : integer);

procedure test_deflate(compr: PChar; comprLen: longint);

procedure test_inflate(compr: PChar;
                       comprLen: longint;
       		            uncompr: PChar;
                       uncomprLen: longint);

procedure test_large_deflate(compr : PChar;
                             comprLen: longint;
                             uncompr: PChar;
                             uncomprLen : longint);

procedure test_large_inflate(compr: PChar;
                             comprLen: longint;
		                          uncompr : PChar;
                             uncomprLen: longint);

procedure test_flush(compr: PChar; comprLen: longint);

procedure test_sync(compr: PChar;
                    comprLen : longint;
		                 uncompr: PChar;
                    uncomprLen : longint);

procedure test_dict_deflate(compr: PChar; comprLen: longint);

procedure test_dict_inflate(compr: PChar;
                            comprLen: longint;
		                         uncompr : PChar;
                            uncomprLen : longint);

implementation

uses exUnit1, zlib, SysUtils;

procedure Check_Err(err: integer; const Msg : string);
begin
  if (err <> Z_OK) then
  begin
    Form1.AddToMemo(Format('!!! %s error: %d', [msg, err]));
  end;
end;

///* ===========================================================================
// * Test compress() and uncompress()
// */
procedure test_compress(compr: PChar; comprLen: longint;
		            uncompr : PChar; uncomprLen: longint);
var
    err: integer;
    len : longint;

begin
  Form1.AddToMemo('--- Test Compress ---');
    len := strlen(hello) + 1;
    err := compress(compr, comprLen, hello, len);
    CHECK_ERR(err, 'compress');

    strcopy(uncompr, 'garbage');

    err := uncompress(uncompr, uncomprLen, compr, comprLen);
    CHECK_ERR(err, 'uncompress');

    if (strcomp(uncompr, hello) <> 0) then
      Form1.AddToMemo('bad uncompress')
    else
      Form1.AddToMemo(Format('uncompress(): %s', [uncompr]));
end;

///* ===========================================================================
// * Test read/write of .gz files
// */
procedure test_gzio(outname, inname : PChar;
		            uncompr : PChar; uncomprLen : integer);
var
    err : integer;
    len : integer;
    gzfile : tgzFile;

begin
  Form1.AddToMemo('--- Test GZIO ---');
    len := strlen(hello) + 1;

    gzfile := gzopen(outname, 'wb');
    if (gzfile = nil) then
      Form1.AddToMemo('gzopen error');

    if (gzwrite(gzfile, hello, len) <> len) then
      Form1.AddToMemo(Format('gzwrite err: %s', [gzerror(gzfile, @err)]));

    gzclose(gzfile);

    gzfile := gzopen(inname, 'rb');
    if (gzfile = nil) then
      Form1.AddToMemo('gzopen error');

    strcopy(uncompr, 'garbage');

    uncomprLen := gzread(gzfile, uncompr, uncomprLen);
    if (uncomprLen <> len) then
      Form1.AddToMemo(Format('gzread err: %s', [gzerror(gzfile, @err)]));

    gzclose(gzfile);

    if (strcomp(uncompr, hello) <> 0) then
      Form1.AddToMemo('bad gzread')
    else
      Form1.AddToMemo(Format('gzread(): %s', [uncompr]));

end;

///* ===========================================================================
// * Test deflate() with small buffers
// */
procedure test_deflate(compr: PChar; comprLen: longint);
var
    c_stream: z_stream; //* compression stream
    err : integer;
    len : integer;
begin
  Form1.AddToMemo('--- Test Deflate ---');
  len := strlen(hello) + 1;

  c_stream.zalloc := nil;
  c_stream.zfree := nil;
  c_stream.opaque := nil;

  err := deflateInit(c_stream, Z_DEFAULT_COMPRESSION);
  CHECK_ERR(err, 'deflateInit');
  c_stream.next_in  := hello;
  c_stream.next_out := compr;

  while (c_stream.total_in <> len) and (c_stream.total_out < comprLen) do
  begin
    c_stream.avail_out := 1; //* force small buffers
    c_stream.avail_in := 1;
    err := deflate(c_stream, Z_NO_FLUSH);
    CHECK_ERR(err, 'deflate');
  end;
  { Finish the stream, still forcing small buffers: }
  while true do
  begin
    c_stream.avail_out := 1;
    err := deflate(c_stream, Z_FINISH);
    if (err = Z_STREAM_END) then break;
    CHECK_ERR(err, 'deflate');
  end;

  err := deflateEnd(c_stream);
  CHECK_ERR(err, 'deflateEnd');
end;

///* ===========================================================================
// * Test inflate() with small buffers
// */
procedure test_inflate(compr: PChar;
                       comprLen: longint;
       		            uncompr: PChar;
                       uncomprLen: longint);
var
  err : integer;
  d_stream : z_stream; //* decompression stream
begin
  Form1.AddToMemo('--- Test Inflate With Small Buffers ---');
  strcopy(uncompr, 'garbage');

  d_stream.zalloc := alloc_func(nil);
  d_stream.zfree := free_func(nil);
  d_stream.opaque := nil;

  err := inflateInit(d_stream);
  CHECK_ERR(err, 'inflateInit');

  d_stream.next_in  := compr;
  d_stream.next_out := uncompr;

  while (d_stream.total_out < uncomprLen) and (d_stream.total_in < comprLen) do
  begin
    d_stream.avail_out := 1; //* force small buffers
    d_stream.avail_in := 1;
    err := inflate(d_stream, Z_NO_FLUSH);
    if (err = Z_STREAM_END) then break;
    CHECK_ERR(err, 'inflate');
  end;

  err := inflateEnd(d_stream);
  CHECK_ERR(err, 'inflateEnd');

  if (strcomp(uncompr, hello) <> 0 ) then
    Form1.AddToMemo('bad inflate')
  else
    Form1.AddToMemo(Format('inflate(): %s', [uncompr]));
end;

///* ===========================================================================
// * Test deflate() with large buffers and dynamic change of compression level
// */
procedure test_large_deflate(compr : PChar;
                             comprLen: longint;
                             uncompr: PChar;
                             uncomprLen : longint);
var
  c_stream: z_stream; //* compression stream
  err : integer;
begin
  Form1.AddToMemo('--- Test Deflate with Large Buffers ---');
  c_stream.zalloc := nil;
  c_stream.zfree := nil;
  c_stream.opaque := nil;

  err := deflateInit(c_stream, Z_BEST_SPEED);
  CHECK_ERR(err, 'deflateInit');

  c_stream.next_out := compr;
  c_stream.avail_out := comprLen;

  { At this point, uncompr is still mostly zeroes, so it should compress
    very well:  }

  c_stream.next_in := uncompr;
  c_stream.avail_in := uncomprLen;
  err := deflate(c_stream, Z_NO_FLUSH);
  CHECK_ERR(err, 'deflate');
  if (c_stream.avail_in <> 0) then
      Form1.AddToMemo('deflate not greedy');

  { Feed in already compressed data and switch to no compression: }
  deflateParams(c_stream, Z_NO_COMPRESSION, Z_DEFAULT_STRATEGY);
  c_stream.next_in := compr;
  c_stream.avail_in := comprLen div 2;
  err := deflate(c_stream, Z_NO_FLUSH);
  CHECK_ERR(err, 'deflate');

  { Switch back to compressing mode: }
  deflateParams(c_stream, Z_BEST_COMPRESSION, Z_FILTERED);
  c_stream.next_in := uncompr;
  c_stream.avail_in := uncomprLen;
  err := deflate(c_stream, Z_NO_FLUSH);
  CHECK_ERR(err, 'deflate');

  err := deflate(c_stream, Z_FINISH);
  if (err <> Z_STREAM_END) then
      Form1.AddToMemo('deflate should report Z_STREAM_END');

  err := deflateEnd(c_stream);
  CHECK_ERR(err, 'deflateEnd');

end;

///* ===========================================================================
// * Test inflate() with large buffers
// */
procedure test_large_inflate(compr: PChar;
                             comprLen: longint;
		                          uncompr : PChar;
                             uncomprLen: longint);
var
    err : integer;
    d_stream : z_stream; //* decompression stream
begin
  Form1.AddToMemo('--- Test Inflate with Large Buffers ---');
  strcopy(uncompr, 'garbage');

  d_stream.zalloc := nil;
  d_stream.zfree := nil;
  d_stream.opaque := nil;

  err := inflateInit(d_stream);
  CHECK_ERR(err, 'inflateInit');

  d_stream.next_in  := compr;
  d_stream.avail_in := comprLen;

  while true do
  begin
      d_stream.next_out := uncompr;            //* discard the output
      d_stream.avail_out := uncomprLen;
      err := inflate(d_stream, Z_NO_FLUSH);
      if (err = Z_STREAM_END) then break;
      CHECK_ERR(err, 'large inflate');
  end;

  err := inflateEnd(d_stream);
  CHECK_ERR(err, 'inflateEnd');

  if (d_stream.total_out <> 2 * uncomprLen + comprLen div 2) then
      Form1.AddToMemo(Format('bad large inflate: %d', [d_stream.total_out]))
  else
      Form1.AddToMemo('large_inflate(): OK');
end;

///* ===========================================================================
// * Test deflate() with full flush
// */
procedure test_flush(compr: PChar; comprLen: longint);
var
    c_stream : z_stream; //* compression stream
    err : integer;
    len : integer;
begin
  Form1.AddToMemo('--- Test Flush ---');
  len := strlen(hello) + 1;

  c_stream.zalloc := nil;
  c_stream.zfree := nil;
  c_stream.opaque := nil;

  err := deflateInit(c_stream, Z_DEFAULT_COMPRESSION);
  CHECK_ERR(err, 'deflateInit');

  c_stream.next_in  := hello;
  c_stream.next_out := compr;
  c_stream.avail_in := 3;
  c_stream.avail_out := comprLen;
  err := deflate(c_stream, Z_FULL_FLUSH);
  CHECK_ERR(err, 'deflate');

  inc(compr[3]); //* force an error in first compressed block
  c_stream.avail_in := len - 3;

  err := deflate(c_stream, Z_FINISH);
  if (err <> Z_STREAM_END) then
      CHECK_ERR(err, 'deflate');

  err := deflateEnd(c_stream);
  CHECK_ERR(err, 'deflateEnd');
end;

///* ===========================================================================
// * Test inflateSync()
// */
procedure test_sync(compr: PChar;
                    comprLen : longint;
		                 uncompr: PChar;
                    uncomprLen : longint);
var
    err : integer;
    d_stream : z_stream; //* decompression stream
begin
  Form1.AddToMemo('--- Test Sync ---');
  strcopy(uncompr, 'garbage');

  d_stream.zalloc := nil;
  d_stream.zfree := nil;
  d_stream.opaque := nil;

  err := inflateInit(d_stream);
  CHECK_ERR(err, 'inflateInit');

  d_stream.next_in  := compr;
  d_stream.next_out := uncompr;
  d_stream.avail_in := 2; //* just read the zlib header
  d_stream.avail_out := uncomprLen;

  inflate(d_stream, Z_NO_FLUSH);
  CHECK_ERR(err, 'inflate');

  d_stream.avail_in := comprLen - 2;   //* read all compressed data
  err := inflateSync(d_stream);           //* but skip the damaged part
  CHECK_ERR(err, 'inflateSync');

  err := inflate(d_stream, Z_FINISH);
  if (err <> Z_DATA_ERROR) then
      Form1.AddToMemo('inflate should report DATA_ERROR');
      //* Because of incorrect adler32

  err := inflateEnd(d_stream);
  CHECK_ERR(err, 'inflateEnd');

  Form1.AddToMemo(Format('after inflateSync(): hel%s', [uncompr]));
end;

///* ===========================================================================
// * Test deflate() with preset dictionary
// */
procedure test_dict_deflate(compr: PChar; comprLen: longint);
var
  c_stream : z_stream; //* compression stream
  err : integer;
begin
  Form1.AddToMemo('--- Test Dictionary Deflate ---');
  c_stream.zalloc := nil;
  c_stream.zfree := nil;
  c_stream.opaque := nil;

  err := deflateInit(c_stream, Z_BEST_COMPRESSION);
  CHECK_ERR(err, 'deflateInit');

  err := deflateSetDictionary(c_stream,
          dictionary, sizeof(dictionary));
  CHECK_ERR(err, 'deflateSetDictionary');

  dictId := c_stream.adler;
  c_stream.next_out := compr;
  c_stream.avail_out := comprLen;

  c_stream.next_in := hello;
  c_stream.avail_in := strlen(hello) + 1;

  err := deflate(c_stream, Z_FINISH);
  if (err <> Z_STREAM_END) then
      Form1.AddToMemo('deflate should report Z_STREAM_END');

  err := deflateEnd(c_stream);
  CHECK_ERR(err, 'deflateEnd');
end;

///* ===========================================================================
// * Test inflate() with a preset dictionary
// */
procedure test_dict_inflate(compr: PChar;
                            comprLen: longint;
		                         uncompr : PChar;
                            uncomprLen : longint);
var
  err : integer;
  d_stream : z_stream; //* decompression stream
begin
  Form1.AddToMemo('--- Test Dictionary Inflate ---');
  strcopy(uncompr, 'garbage');

  d_stream.zalloc := nil;
  d_stream.zfree := nil;
  d_stream.opaque := nil;

  err := inflateInit(d_stream);
  CHECK_ERR(err, 'inflateInit');

  d_stream.next_in  := compr;
  d_stream.avail_in := comprLen;

  d_stream.next_out := uncompr;
  d_stream.avail_out := uncomprLen;

  while true do
  begin
    err := inflate(d_stream, Z_NO_FLUSH);
    if (err = Z_STREAM_END) then break;
    if (err = Z_NEED_DICT) then
         if (d_stream.adler <> dictId) then
         begin
            Form1.AddToMemo('unexpected dictionary');
            exit;
         end;
    err := inflateSetDictionary(d_stream, dictionary, sizeof(dictionary));
    CHECK_ERR(err, 'inflate with dict');
    if (err <> Z_OK) then break;
  end;

  err := inflateEnd(d_stream);
  CHECK_ERR(err, 'inflateEnd');

  if (strcomp(uncompr, hello) <> 0) then
    Form1.AddToMemo('bad inflate with dict')
  else
    Form1.AddToMemo(format('inflate with dictionary: %s', [uncompr]));

  end;

end.
